/* Copyright 2006 Sun Microsystems, Inc.  All rights reserved. */

/*
**  sim.c	- Simple Shade analyzer to simulate target application
**
**	This file defines a simple Shade analyzer that simply simulates
**	the target application.  It does no tracing whatsoever.
*/

#pragma ident "@(#)Shade and Spixtools for Solaris 10 v1.0.1"

#include <stdio.h>
#include <string.h>
#include <assert.h>
#include <shade.h>
#include <shade_anal.h>
#include "getopt.h"

/*
 * Define a standard trace record.
 */
struct shade_trace_s {SHADE_TRACE};


const char	shadeuser_analversion[] = "sim " "v1.0.1";

const shade_options_t shadeuser_opts[]={
    {0, "", NULL, NO_PARAM}
};

/*
**  Perform initializations specific to the 'sim' analyzer.  Return zero
**  on success, non-zero on failure.
*/
int shadeuser_initialize(
	int     	argc,
	char ** 	argv,
	char ** 	envp)
{
	spix_bool_t	ok;
	int     	i, j;
    int                 sim_optnum;
    shade_option_val_t  sim_opt_value;

        char            *pp;

        

	ok = TRUE;
	for (i = j = 1;  i < argc;  i++) {
        
        sim_optnum = shade_getoptv(argc, argv, &i, &j, shadeuser_opts, &sim_opt_value);
    
        if(sim_optnum!=-1){
            shade_error("Unknown option '%s'.", argv[i]);
            ok = FALSE;
        }
	}

	if (!ok) {
		shade_usage();
		return(1);
	}

	return(0);
}


/*
**  Run and trace an application that is loaded into Shade.
*/
int shadeuser_analyze(void)
{
	shade_trace_t	tr;


	/*
	 * Since we are not tracing any instructions, shade_run() only
	 * return if the application terminates.  (It also returns if the
	 * application forks or calls exec(), but shadeuser_analyze() is called again in
	 * that event.)
	 */
	shade_run(&tr, 1);

	return(0);
}

/*
 * "sim" does not utilize "shadeuser_report" API as it is not
 *  supposed to produce any kinds of reports
 */

/*
**  Perform termination activities after all applications have been executed.
*/
int shadeuser_terminate(
	int	ret)
{
	int	val;

	/*
	 * Exit with the termination status from the executed application.
	 */
	switch (shade_appstatus(&val))
	{
	case SHADE_STATUS_EXITED:
		return(val);

	case SHADE_STATUS_SIGNALED:
		return(128 + val);

	case SHADE_STATUS_NOAPP:
		return(0);

	case SHADE_STATUS_LOADED:
	case SHADE_STATUS_RUNNING:
	default:
		assert(0);
	}

}


/*
**  Print a usage message for the command line options that are specific
**  to the 'sim' analyzer.
*/
void shadeuser_analusage(void)
{
    /*
	fprintf(stderr, "%-20s %s\n", "-s <signal>",
		"Note : this option is not functional for \"sim\".");*/
	/* nothing to do */

    shade_print_opt_info(shadeuser_opts, 0);
}

void
shadeuser_report(int reason, void* ptr) 
{

  return;

}

