/* Copyright 2006 Sun Microsystems, Inc.  All rights reserved. */

/*
**  mpshade_sparcv9.h	- Interface to MPShade for SPARC V9
**
**	MPShade is multi-process extension that layers on top of Shade.
**	A single MPShade analyzer process can collect trace data from many
**	MPShade tracer processes.  Each tracer runs a single application
**	under Shade and feeds trace records to the shared analyzer.
**
**	This header defines the SPARC V9 specific interfaces for MPShade
**	analyzers and tracers.  See <mpshade.h> for a description of the
**	processor independent MPShade interfaces.
*/

#ifndef MPSHADE_SPARCV9_H
#define MPSHADE_SPARCV9_H

#pragma ident "@(#)Shade and Spixtools for Solaris 10 v1.0.1"

#include <mpshade.h>
#include <spix_sparc.h>
#include <shade_sparcv9.h>

#ifdef __cplusplus
extern "C" {
#endif

/*
 * Additional SPARC V9 specific trace record sizes.
 */
#define MPSHADE_SPARCV9_TRSZ_TRACE      	((mpshade_trsz_t)8)
#define MPSHADE_SPARCV9_TRSZ_TRACEID    	((mpshade_trsz_t)9)
#define MPSHADE_SPARCV9_TRSZ_REGTRACE   	((mpshade_trsz_t)10)
#define MPSHADE_SPARCV9_TRSZ_REGTRACEID 	((mpshade_trsz_t)11)
#define MPSHADE_SPARCV9_TRSZ_FREGTRACE  	((mpshade_trsz_t)12)
#define MPSHADE_SPARCV9_TRSZ_FREGTRACEID 	((mpshade_trsz_t)13)


/*
 * The definition of a trace record for MPSHADE_SPARCV9_TRSZ_TRACE.
 */
#define MPSHADE_SPARCV9_TRACE   	\
	SHADE_SPARCV9_TRACE

/*
 * The definition of a trace record for MPSHADE_SPARCV9_TRSZ_TRACEID.
 */
#define MPSHADE_SPARCV9_TRACEID   	\
	SHADE_SPARCV9_TRACE     	\
	mpshade_id_t	tr_trid;

/*
 * The definition of a trace record for MPSHADE_SPARCV9_TRSZ_REGTRACE.
 */
#define MPSHADE_SPARCV9_REGTRACE	\
	SHADE_SPARCV9_REGTRACE

/*
 * The definition of a trace record for MPSHADE_SPARCV9_TRSZ_REGTRACEID.
 */
#define MPSHADE_SPARCV9_REGTRACEID	\
	SHADE_SPARCV9_REGTRACE  	\
	mpshade_id_t	tr_trid;

/*
 * The definition of a trace record for MPSHADE_SPARCV9_TRSZ_FREGTRACE.
 */
#define MPSHADE_SPARCV9_FREGTRACE	\
	SHADE_SPARCV9_FREGTRACE

/*
 * The definition of a trace record for MPSHADE_SPARCV9_TRSZ_FREGTRACEID.
 */
#define MPSHADE_SPARCV9_FREGTRACEID	\
	SHADE_SPARCV9_FREGTRACE 	\
	mpshade_id_t	tr_trid;


/*
 * Additional standard-tracer layer analyzer interfaces for creating
 * SPARC V9 specific instruction sets.
 */
mpshade_iset_t *mpshade_iset_newop(spix_sparc_iop_t, ...);
mpshade_iset_t *mpshade_iset_addop(mpshade_iset_t *, spix_sparc_iop_t, ...);
mpshade_iset_t *mpshade_iset_newtype(spix_sparc_itype_t, ...);
mpshade_iset_t *mpshade_iset_addtype(mpshade_iset_t *, spix_sparc_itype_t, ...);

#ifdef __cplusplus
}
#endif

#endif /*MPSHADE_SPARCV9_H*/
