/* Copyright 2006 Sun Microsystems, Inc.  All rights reserved. */

/*
**  shade_anal.h	- Interface to common interface for Shade analyzers
**
**	This file defines the interface to the optional common user
**	interface for Shade analyzers.  Analyzers using this interface
**	should include "shade_anal.o" on their link line before "libshade.a".
**	See the shade_anal(3s) manpage for details.
*/

#ifndef SHADE_ANAL_H
#define SHADE_ANAL_H

#pragma ident "@(#)Shade and Spixtools for Solaris 10 v1.0.1"

#include <shade.h>

extern FILE* mapfile_handle;

/*
 * Entry point for Shade analyzers.
 * Shade library provides the default version of this routine. 
 * See the shade_anal(3s) manpage for details.
 */
int	shadeuser_main(int, char **, char **);

/*
 * The analyzer must define the following global variable.
 */
extern const char	shadeuser_analversion[];

/*
 * Analyzer must define the following routines and variables 
 * if using default shadeuser_main(): 
 */

/* Analyzer-specific initialization */
int     	shadeuser_initialize(int, char **, char **);

/* Analysis */
int     	shadeuser_analyze(void);

/* Report statistics, codes for reason parameter: */

/* Shade is about to terminate, data is return code */
# define SHADEUSER_REPORT_TERMINATE         0x0
/* shadeuser_analyze() loop done,  data is return code */
# define SHADEUSER_REPORT_APP_DONE          0x1
/* signal received,  data is signal number */
# define SHADEUSER_REPORT_SIGNAL            0x2
/* checkpoint request received,  data is reserved */
# define SHADEUSER_REPORT_CHECKPOINT        0x3

/* entries below are reserved up to USER1 */ 
# define SHADEUSER_REPORT_USER1         0x100

void     	shadeuser_report(int reason, void* data);

/* Analyzer-specific finalization */
int     	shadeuser_terminate(int);

/* Usage printout, describe your options here */
void    	shadeuser_analusage(void);

/*
 *  Old names for the above user-definable routines.
 *  Provided for compatibility only - do not use them in new programs !
 */
#define shade_main        shadeuser_main
#define shade_initialize  shadeuser_initialize
#define shade_analyze     shadeuser_analyze
#define shade_terminate   shadeuser_terminate
#define shade_analusage   shadeuser_analusage
#define shade_analversion shadeuser_analversion

#endif /*SHADE_ANAL*/
