/* Copyright 2006 Sun Microsystems, Inc.  All rights reserved. */

/*
**  shade_sparcv9.h	- Public Shade interfaces for SPARC V9
**
**	This header defines the SPARC V9 specific public interfaces for
**	Shade analyzers.  See <shade.h> for a description of the processor
**	independent Shade analyzer interfaces.
*/

#ifndef SHADE_SPARCV9_H
#define SHADE_SPARCV9_H

#pragma ident "@(#)Shade and Spixtools for Solaris 10 v1.0.1"

#define	 SPIX_ADDR	64
#include <shade.h>
#include <spix_sparc.h>

#ifdef __cplusplus
extern "C" {
#endif


/*
 * Additional processor specific trace control parameters.
 */
#define SHADE_SPARCV9_TRCTL_I		((shade_trctl_t)16)
#define SHADE_SPARCV9_TRCTL_RS1		((shade_trctl_t)17)
#define SHADE_SPARCV9_TRCTL_RS2		((shade_trctl_t)18)
#define SHADE_SPARCV9_TRCTL_RD		((shade_trctl_t)19)
#define SHADE_SPARCV9_TRCTL_FRS1	((shade_trctl_t)20)
#define SHADE_SPARCV9_TRCTL_FRS2	((shade_trctl_t)21)
#define SHADE_SPARCV9_TRCTL_FRD		((shade_trctl_t)22)

  /* XXXX TS */
#define SHADE_SPARCV9_TRCTL_TICK        ((shade_trctl_t)23)


/*
 * The definition of a SPARC V9 integer register.
 */
typedef union {
	spix_int32_t	ii[2];
	spix_uint32_t	uu[2];
	spix_int64_t	ix;
	spix_uint64_t	ux;
} shade_sparcv9reg_t;


/*
 * The definition of a SPARC V9 floating point register.
 */
typedef union {
	spix_int32_t	i, ii[2], iiii[4];
	spix_uint32_t	u, uu[2], uuuu[4];
	spix_int64_t	ix, ixx[2];
	spix_uint64_t	ux, uxx[2];
	spix_real32_t	s, ss[2], ssss[4];
	spix_real64_t	d, dd[2];
#ifdef SPIX_R128
	spix_real128_t	q;
#endif
} shade_sparcv9freg_t;



/*
 * The definition of the initial fields in any SPARC V9 trace record.
 */
#define SHADE_SPARCV9_TRACE			\
	SHADE_TRACE				\
	spix_sparc_inst_t	tr_i;		\
	unsigned		tr_reserved1;



/*
 * The definition of the initial fields in SPARC V9 trace record that
 * includes integer register values.
 */
#define SHADE_SPARCV9_REGTRACE			\
	SHADE_SPARCV9_TRACE			\
	shade_sparcv9reg_t	tr_rs1;		\
	shade_sparcv9reg_t	tr_rs2;		\
	shade_sparcv9reg_t	tr_rd;


/*
 * The definition of the initial fields in SPARC V9 trace record that
 * includes integer and FP register values.
 */
#define SHADE_SPARCV9_FREGTRACE			\
	SHADE_SPARCV9_REGTRACE			\
	shade_sparcv9freg_t	tr_frs1;	\
	shade_sparcv9freg_t	tr_frs2;	\
	shade_sparcv9freg_t	tr_frd;



	/* XXX  TS  */

/*
 * Experimental format which contains addtional field (TICK register - time stamp)
 */
#define SHADE_SPARCV9_TRACE_TS			\
	SHADE_SPARCV9_FREGTRACE				\
       	unsigned long long      tr_tick;
       
	/* XXXX */


/*
 * The set of all SPARC V9 floating point registers.
 */
typedef union {
	spix_int32_t	i[64];
	spix_uint32_t	u[64];
	spix_real32_t	s[32];
	spix_real64_t	d[32];
#ifdef SPIX_R128
	spix_real128_t	q[16];
#endif
} shade_sparcv9fregset_t;


/*
 * The set of all non-privileged SPARC V9 registers.
 */
struct shade_regs_s {
	spix_int32_t		sh_tid;
	spix_int32_t		sh_y;
	shade_sparcv9reg_t	sh_r[32];
	spix_uint8_t		sh_ccr;
	spix_uint8_t		sh_asi;
	spix_uint16_t		reserved;

	shade_sparcv9fregset_t	sh_fr;
	shade_sparcv9reg_t	sh_xfsr;
	shade_sparcv9reg_t	sh_gsr;
};


/*
 * Additional interfaces for creating instruction sets from opcode values.
 */
shade_iset_t *	shade_iset_newop(spix_sparc_iop_t, ...);
shade_iset_t *	shade_iset_addop(shade_iset_t *, spix_sparc_iop_t, ...);
shade_iset_t *	shade_iset_newtype(spix_sparc_itype_t, ...);
shade_iset_t *	shade_iset_addtype(shade_iset_t *, spix_sparc_itype_t, ...);


#ifdef __cplusplus
}
#endif

#endif /*SHADE_SPARCV9_H*/
