/* Copyright 2006 Sun Microsystems, Inc.  All rights reserved. */

/*
**	spix counts file format:
**
**	header:
**		SCHdr struct
**		text size and check sum of pre-spix a.out are included 
**		to ensure consistency for spixstats, sdas, etc.
**
**	basic block lengths:
**		1 spix_uint8_t per block containing # instructions in block.
**		large blocks must be split.  padded to 4 byte boundary.
**
**	basic block execution counts:
**		2 (adjacent) spix_sc_t's per block:
**		for uncounted blocks (e.g. data), both counters are SC_BADCNT.
**		for blocks ending in a branch, 1st is taken count,
**		    2nd is not-taken count.
**		for other blocks, 1st is execution count, 2nd is unused.
*/

#ifndef SPIXCOUNTS_H
#define SPIXCOUNTS_H

#pragma ident "@(#)Shade and Spixtools for Solaris 10 v1.0.1"

#include <spixtypes.h>

#ifdef __cplusplus
extern "C" {
#endif


#define SPIX_SC_BADCNT	(~ (spix_sc_t) 0LL)

#define SPIX_SC_MAGIC32	(0x62620011)

#define SPIX_SC_MAGIC	(0x62620022)


typedef spix_int64_t	spix_sc_t;

/* XXX To fix HIBIT bug in SADD (SL) */
typedef spix_uint64_t   spix_scu_t;


typedef struct {
	spix_uint32_t	sc_magic;	/* SPIX_SC_MAGIC */
	spix_uint32_t	sc_tsize;	/* size of text segment (bytes) */
	spix_uint32_t	sc_tsum; 	/* text segment check_sum() */
	spix_uint32_t	sc_nbb;  	/* # basic blocks */
} spix_schdr_t;


#ifdef __cplusplus
}
#endif

#endif /*SPIXCOUNTS_H*/
