/* Copyright 2006 Sun Microsystems, Inc.  All rights reserved. */

/*
**  spixtypes.h 	- Basic types exported from spix library
**
**	This file defines basic types used by interfaces to the spix
**	library.
*/

#ifndef SPIXTYPES_H
#define SPIXTYPES_H

#pragma ident "@(#)Shade and Spixtools for Solaris 10 v1.0.1"

#ifdef __cplusplus
extern "C" {
#endif


typedef signed char     	spix_int8_t;
typedef unsigned char   	spix_uint8_t;
typedef short           	spix_int16_t;
typedef unsigned short  	spix_uint16_t;
typedef int             	spix_int32_t;
typedef unsigned        	spix_uint32_t;

#ifndef SPIX_NOLONGLONG
#define SPIX_I64
typedef long long       	spix_int64_t;
typedef unsigned long long	spix_uint64_t;
#endif /*SPIX_NOLONGLONG*/


typedef float            	spix_real32_t;
typedef double            	spix_real64_t;

#ifndef SPIX_NOLONGDOUBLE
#define SPIX_R128
typedef long double     	spix_real128_t;
#endif /*SPIX_NOLONGDOUBLE*/


typedef spix_uint32_t   	spix_addr32_t;
#ifdef SPIX_I64
typedef spix_uint64_t   	spix_addr64_t;
#endif

#if SPIX_ADDR == 32
    typedef spix_addr32_t	spix_addr_t;
#elif SPIX_ADDR == 64
    typedef spix_addr64_t	spix_addr_t;
#endif


#ifdef FALSE
#   if FALSE != 0
#       error "Conflict with constant FALSE"
#   endif
#   undef FALSE
#endif

#ifdef TRUE
#   if TRUE != 1
#       error "Conflict with constant TRUE"
#   endif
#   undef TRUE
#endif

#define FALSE	0
#define TRUE	1

typedef unsigned	spix_bool_t;

/* Spix5 types. */

/* signed or unsigned, == 8 bits */
#ifndef INT8
#define INT8 char
typedef INT8 int8;
#endif

/* unsigned, == 8 bits */
#ifndef UINT8
#define UINT8 unsigned char
typedef UINT8 uint8;
#endif

/* signed, == 16 bits */
#ifndef INT16
#define INT16 short
typedef INT16 int16;
#endif

/* unsigned, == 16 bits */
#ifndef UINT16
#define UINT16 unsigned short
typedef UINT16 uint16;
#endif

/* signed, == 32 bits */
#ifndef INT32
#define INT32 int
typedef INT32 int32;
#endif

/* unsigned, == 32 bits */
#ifndef UINT32
#define UINT32 unsigned
typedef UINT32 uint32;
#endif

/* signed, == 64 bits */
#ifndef INT64
#define INT64 long long
typedef INT64 int64;
#endif

/* unsigned, == 64 bits */
#ifndef UINT64
#define UINT64 unsigned long long
typedef UINT64 uint64;
#endif

#ifndef REAL32
#define REAL32 float
typedef REAL32 real32;
#endif

#ifndef REAL64
#define REAL64 double
typedef REAL64 real64;
#endif

#ifndef REAL128
#define REAL128 long double
typedef REAL128 real128;
#endif

#ifdef __cplusplus
}
#endif

#endif /*SPIXTYPES_H*/
